package ch.pboos.android.SleepTimer;

import java.util.Calendar;
import java.util.List;

import com.nullwire.trace.ExceptionHandler;

import android.app.Activity;
import android.app.ActivityManager;
import android.app.AlarmManager;
import android.app.Dialog;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.Html;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.Toast;

public class SleepTimer extends Activity {
	public static final String PREFS_OWN_SERVICE = "ownService";
	public static final String PREFS_MINUTES = "minutes";
	public static final String INTENT_EXTRA_STOP_MINUTES = "stopMinutes";
	public static final int TIME_BETWEEN_INFO = 60; // TODO: set time again to 60
	private static final int SLEEP_TIMER_NOTIFICATION_ID = 57339;
	protected static final String PREFS_MUSIC_PLAYER = "pref_musicapp";
	
	private Button buttonStartStop;
	private Button buttonStartPlayer;
	private Button buttonSetMinutes;
	private int sleep_minutes;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        ExceptionHandler.register(this, "http://pboos.ch/bugs/server.php");
        
        setContentView(R.layout.main);
                
        buttonStartStop = (Button)findViewById(R.id.ButtonStart);
        buttonStartPlayer = (Button)findViewById(R.id.Button_StartPlayer);
        buttonSetMinutes = (Button)findViewById(R.id.ButtonTime);
        
        SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(this);
        sleep_minutes = settings.getInt(PREFS_MINUTES, 5);
        setButtonMinutes(sleep_minutes);
        
        buttonStartPlayer.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(SleepTimer.this);
		        String musicPlayerPackage = settings.getString(PREFS_MUSIC_PLAYER, "");
				Intent musicPlayerIntent = getPackageManager().getLaunchIntentForPackage(musicPlayerPackage);
		    	if(musicPlayerIntent==null){
		    		 Toast mToast = Toast.makeText(SleepTimer.this, SleepTimer.this.getResources().getString(R.string.pref_music_player_not_set),
			                    Toast.LENGTH_LONG);
			            mToast.show();
		    	} else {
					Log.d("START", musicPlayerPackage);
					try {
						startActivity(musicPlayerIntent);
					} catch(ActivityNotFoundException e){
						if(musicPlayerPackage.equals("com.android.music")){
							musicPlayerIntent = new Intent("com.android.music.PLAYBACK_VIEWER");
							startActivity(musicPlayerIntent);
						} else {
							throw e;
						}
					}
		    	}
			}
		});
        buttonSetMinutes.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				Intent i = new Intent(SleepTimer.this, SetTimeDialog.class);
				i.putExtra("minutes", sleep_minutes);
				startActivityForResult(i, 0);
			}
		});
        
        buttonStartStop.setOnClickListener(new OnClickListener(){
			@Override
			public void onClick(View v) {
				if(!SleepTimerStatus.isRunning(SleepTimer.this)){
					startSleepTimer();
				} else {
					stopSleepTimer();
				}
			}
        });
    }
    
    private void stopSleepTimer() {
		Intent intent = new Intent(SleepTimer.this, StopMusicReceiver.class);
		intent.putExtra(INTENT_EXTRA_STOP_MINUTES, 0);
		PendingIntent sender = PendingIntent.getBroadcast(SleepTimer.this,
		        0, intent, PendingIntent.FLAG_CANCEL_CURRENT);
		// And cancel the alarm.
		AlarmManager am = (AlarmManager)getSystemService(ALARM_SERVICE);
		am.cancel(sender);
		stopNotification(SleepTimer.this);
		
		SleepTimerStatus.setRunning(this, false);
		setStartStopButtonText();
		
		StopMusicReceiver.setShakeInactive(SleepTimer.this);
	}


	public static void setNotificationMinutes(Context context, String minutes) {
		setNotification(context, minutes+" "+context.getResources().getString(R.string.notify_minutes_left), minutes+" "+context.getResources().getString(R.string.notify_minutes_left_until_sleep));
	}
    
	public static void setNotification(Context context, String text, String additional){
		String ns = Context.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) context.getSystemService(ns);
		
		// Short Information
		int icon = R.drawable.sleep_icon;
		CharSequence tickerText = text;
		long when = System.currentTimeMillis();

		Notification notification = new Notification(icon, tickerText, when);
		notification.flags = notification.flags | Notification.FLAG_ONGOING_EVENT;
		
		// Extended Information
		CharSequence contentTitle = "Sleep Timer";
		CharSequence contentText = additional;
		Intent notificationIntent = new Intent(context, SleepTimer.class);
		PendingIntent contentIntent = PendingIntent.getActivity(context, 0, notificationIntent, 0);
		notification.setLatestEventInfo(context, contentTitle, contentText, contentIntent);
		
		// Pass to NotificationManager
		mNotificationManager.notify(SLEEP_TIMER_NOTIFICATION_ID, notification);
	}
	
	public static void stopNotification(Context context){
		String ns = Context.NOTIFICATION_SERVICE;
		NotificationManager mNotificationManager = (NotificationManager) context.getSystemService(ns);
		mNotificationManager.cancel(SLEEP_TIMER_NOTIFICATION_ID);
	}
	
	/* Creates the menu items */
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
	    inflater.inflate(R.menu.options_menu, menu);
	    return true;
	}
	
	/* Handles item selections */
	public boolean onOptionsItemSelected(MenuItem item) {
	    switch (item.getItemId()) {
	    case R.id.menu_settings:
	    	Intent settingsIntent = new Intent(this, Preferences.class);
	    	startActivity(settingsIntent);
	    	return true;
	    case R.id.menu_changelog:
	    	Intent i = new Intent(this, ChangeLog.class);
	    	startActivity(i);
	        return true;
	    case R.id.menu_feedback:
	    	//Context mContext = getApplicationContext();
	    	final Dialog dialog = new Dialog(this);

	    	dialog.setContentView(R.layout.feedback_dialog);
	    	dialog.setTitle(getResources().getString(R.string.dialog_feedback_title));	        
	    	Button btnSend = (Button)dialog.findViewById(R.id.Button_send);
	    	btnSend.setOnClickListener(new OnClickListener() {			
				@Override
				public void onClick(View v) {
					String sendText = "";
					CheckBox box = (CheckBox)dialog.findViewById(R.id.CheckBox_services);
					if(box.isChecked()){
						sendText += getResources().getString(R.string.dialog_feedback_player_name)+"<br/><br/>";
						sendText += "Services:<br/>";
						ActivityManager am = (ActivityManager)SleepTimer.this.getSystemService(ACTIVITY_SERVICE);
						List<ActivityManager.RunningServiceInfo> rs = am.getRunningServices(50);
						for (int i=0; i<rs.size(); i++) {
							ActivityManager.RunningServiceInfo rsi = rs.get(i);
							sendText += rsi.process+" - "+rsi.service.getClassName()+"<br/>";
						}
					}
					
					Intent mailIntent = new Intent(Intent.ACTION_SEND);
			    	mailIntent.setType("text/plain");
			    	mailIntent.putExtra(Intent.EXTRA_EMAIL, new String[]{"mail@pboos.ch"});
			    	mailIntent.putExtra(Intent.EXTRA_SUBJECT, "SleepTimer Feedback/Bug/Request");
			    	mailIntent.putExtra(Intent.EXTRA_TEXT, Html.fromHtml(sendText));
			    	startActivity(Intent.createChooser(mailIntent, "Select e-mail application"));
			    	startActivity(mailIntent);
			    	dialog.dismiss();
				}
			});
	    	dialog.show();
	        return true;
	    case R.id.menu_donate:
	    	Intent viewIntent = new Intent("android.intent.action.VIEW", Uri.parse("https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=BN5FVBQUTZGEG&lc=CH&item_name=SleepTimer&currency_code=USD&bn=PP%2dDonationsBF%3abtn_donate_SM%2egif%3aNonHosted"));  
	    	startActivity(viewIntent);
	        return true;
		case R.id.menu_quit:
			this.finish();
			return true;
    	}
	    return false;
	}
	
	
    @Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
    	if(data!=null && data.hasExtra("minutes")){
	    	sleep_minutes = data.getIntExtra("minutes", sleep_minutes);
	    	SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(this);
	    	SharedPreferences.Editor editor = settings.edit();
	        editor.putInt(PREFS_MINUTES, sleep_minutes);
	        editor.commit();
	        setButtonMinutes(sleep_minutes);
    	}
	}
    
    private void setButtonMinutes(int min){
    	buttonSetMinutes.setText(String.format(getResources().getString(R.string.x_minutes),Integer.toString(sleep_minutes)));
    }

	private void startSleepTimer() {
		if(sleep_minutes<1){
			sleep_minutes=1;
			setButtonMinutes(sleep_minutes);
		}
		
		Intent intent = new Intent(SleepTimer.this, StopMusicReceiver.class);
		intent.putExtra(INTENT_EXTRA_STOP_MINUTES, sleep_minutes);
		
		PendingIntent sender = PendingIntent.getBroadcast(SleepTimer.this, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);

		Calendar calendar = Calendar.getInstance();
		calendar.setTimeInMillis(System.currentTimeMillis());
		calendar.add(Calendar.SECOND, TIME_BETWEEN_INFO);

		// Schedule the alarm!
		AlarmManager am = (AlarmManager)getSystemService(ALARM_SERVICE);
		am.set(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), sender);

		// Tell the user about what we did.
		setNotificationMinutes(SleepTimer.this, Integer.toString(sleep_minutes));
		Toast mToast = Toast.makeText(SleepTimer.this, String.format(SleepTimer.this.getResources().getString(R.string.notify_going_to_sleep_in), Integer.toString(sleep_minutes)),
		        Toast.LENGTH_LONG);
		mToast.show();
		
		SleepTimerStatus.setRunning(this, true);
		setStartStopButtonText();
	}

	@Override
	protected void onResume() {
		super.onResume();
		setStartStopButtonText();
	}

	private void setStartStopButtonText() {
		setStartStopButtonText(SleepTimerStatus.isRunning(this));
	}
	
	private void setStartStopButtonText(boolean isRunning){
		Button startStopButton = (Button)findViewById(R.id.ButtonStart);
		if(isRunning){
			startStopButton.setCompoundDrawablesWithIntrinsicBounds(android.R.drawable.ic_delete, 0, 0, 0);
			startStopButton.setText(R.string.button_stop_sleeptimer);
		} else {
			startStopButton.setCompoundDrawablesWithIntrinsicBounds(android.R.drawable.ic_media_play, 0, 0, 0);
			startStopButton.setText(R.string.button_start_sleeptimer);
		}
	}
	
	
}