package ch.pboos.android.SleepTimer;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import ch.pboos.android.SleepTimer.StopConnections.AndroidMediaPlayerStopperServiceConnection;
import ch.pboos.android.SleepTimer.StopConnections.HtcMediaPlayerStopperServiceConnection;

import com.nullwire.trace.ExceptionHandler;

import android.app.ActivityManager;
import android.app.AlarmManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.media.AudioManager;
import android.net.wifi.WifiManager;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.util.Log;

public class StopMusicService extends Service {
	int oldMusicVolumeLevel = 0;
	
	@Override
	public void onStart(Intent intent, int startId) {
		super.onStart(intent, startId);
		
		ExceptionHandler.register(this, "http://pboos.ch/bugs/server.php");
		
		oldMusicVolumeLevel = dimMusicVolume();
		stopMusic();
		additionalStopSettings();
		killDeadServices();
		setVolumeBack(oldMusicVolumeLevel);
		SleepTimerStatus.setRunning(this, false);
	}

	private void additionalStopSettings() {
		SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(this);
		boolean turnOffWifi = settings.getBoolean("pref_stop_wifi", false);
		boolean turnOffBluetooth = settings.getBoolean("pref_stop_bluetooth", false);
		boolean goIntoAirplane = settings.getBoolean("pref_go_airplane", false);
		boolean turnOffNotifications = settings.getBoolean("pref_mute_notifications", false);
		
		if(turnOffWifi)
			turnOffWifi();
		if(turnOffBluetooth)
			turnOffBluetooth();
		if(turnOffNotifications)
			muteNotifications();
		if(goIntoAirplane)
			goIntoAirplaneMode();
	}

	private void turnOffBluetooth() {
		try{
			Object mDevice = this.getSystemService("bluetooth");
			if (mDevice != null) { 
	            Class<?> c = mDevice.getClass(); 
	            Method mEnable = c.getMethod("enable"); 
	            mEnable.setAccessible(true); 
	            Method mDisable = c.getMethod("disable"); 
	            mDisable.setAccessible(true); 
	            Method mIsEnabled = c.getMethod("isEnabled"); 
	            mIsEnabled.setAccessible(true); 
	            
	            if((Boolean) mIsEnabled.invoke(mDevice)){
	            	mDisable.invoke(mDevice);
	            }
	       } 
		} catch (Exception e) {
			Log.d("SleepTimer", "Could not turn off bluetooth: "+e.getCause());
		}
	}

	private void goIntoAirplaneMode() {
		boolean isEnabled = Settings.System.getInt(this.getContentResolver(), Settings.System.AIRPLANE_MODE_ON, 0) == 1;
		if(!isEnabled){
			// toggle airplane mode 
			Settings.System.putInt(this.getContentResolver(),Settings.System.AIRPLANE_MODE_ON, isEnabled ? 0 : 1); 
	
			// Post an intent to reload 
			Intent intent = new Intent(Intent.ACTION_AIRPLANE_MODE_CHANGED); 
			intent.putExtra("state", !isEnabled); 
			sendBroadcast(intent);
		}
	}

	private void muteNotifications() {
		AudioManager manager = (AudioManager)getSystemService(AUDIO_SERVICE);
		manager.setStreamVolume(AudioManager.STREAM_NOTIFICATION, 0, 0);
	}

	private void turnOffWifi() {
		WifiManager mWm = (WifiManager) this.getSystemService(Context.WIFI_SERVICE);
		if(mWm.isWifiEnabled())
			mWm.setWifiEnabled(false);
	}

	private int dimMusicVolume() {
		AudioManager manager = (AudioManager)getSystemService(AUDIO_SERVICE);
		int vol = manager.getStreamVolume(AudioManager.STREAM_MUSIC);
		for(int i = vol-1; i>=0;i--){
			manager.setStreamVolume(AudioManager.STREAM_MUSIC, i, 0);
			try {
				Thread.sleep(700);
			} catch (InterruptedException e) {
			}
		}
		return vol;
	}

	private void stopMusic() {
		List<String> restartMusicServices = getFixSetServices();
		String ownService = getOwnService();

		ActivityManager am = (ActivityManager)this.getSystemService(ACTIVITY_SERVICE);
		List<ActivityManager.RunningServiceInfo> rs = am.getRunningServices(50);
		for (int i=0; i<rs.size(); i++) {
			ActivityManager.RunningServiceInfo rsi = rs.get(i);
						
			String serviceName = rsi.service.getClassName();
			String processName = rsi.process;
			
			if(isHtcPlayer(serviceName))
				stopHtcPlayer();
			if (isAndroidPlayer(serviceName))
				stopAndroidPlayer();
			
			if(restartMusicServices.contains(processName) || (ownService!="" && ownService.equals(processName))){
		    	int index;
		    	if((index = processName.indexOf(":"))>0)
		    		processName = processName.substring(0, index);
		    	
				am.restartPackage(processName);
			}
		}
	}
	
	private boolean isAndroidPlayer(String serviceName) {
		return serviceName.equals("com.android.music.MediaPlaybackService");
	}

	private boolean isHtcPlayer(String serviceName) {
		return serviceName.equals("com.htc.music.MediaPlaybackService");
	}

	private void stopAndroidPlayer() {
		Intent intent = new Intent();
		intent.setClassName("com.android.music", "com.android.music.MediaPlaybackService");
		AndroidMediaPlayerStopperServiceConnection conn = new AndroidMediaPlayerStopperServiceConnection();
		bindService(intent, conn, 0);
		unbindService(conn);
	}

	private void stopHtcPlayer() {
		Intent intent = new Intent();
		intent.setClassName("com.htc.music", "com.htc.music.MediaPlaybackService");
		HtcMediaPlayerStopperServiceConnection conn = new HtcMediaPlayerStopperServiceConnection();
		bindService(intent, conn, 0);
		unbindService(conn);
	}

	private List<String> getFixSetServices() {
		// TODO: Listen in Array in Resources speichern? getResources().getStringArray(R.array....)
		List<String> restartMusicServices = new ArrayList<String>();

		// not in anymore because now again through pause in service
//		restartMusicServices.add("com.android.music");				 		// works
//		restartMusicServices.add("com.htc.music");  						// works
		
		restartMusicServices.add("com.streamfurious.android.free");  		// works
		restartMusicServices.add("com.google.android.apps.listen:remote"); 	// works
		restartMusicServices.add("com.google.android.apps.listen"); 		// (not needed i think, but just for making sure
		restartMusicServices.add("com.imeem.gynoid"); 						// works
		restartMusicServices.add("fm.last.android:remote");					// works
		restartMusicServices.add("fm.last.android:player");					// works
		restartMusicServices.add("org.abrantes.filex");						// Rockon:works
		restartMusicServices.add("org.abrantes.rockonlite");				// works
		restartMusicServices.add("com.tunewiki.lyricplayer.android:player");// works
		restartMusicServices.add("com.tunewiki.lyricplayer.android");
		restartMusicServices.add("com.nsw.android.mediaexplorer");			// works
		restartMusicServices.add("com.mixzing.basic");  					// works
		
		// testing needed to confirm they are correctly stopped
		restartMusicServices.add("com.streamfurious.android.pro");
		restartMusicServices.add("com.pandora.android");
		restartMusicServices.add("mobi.beyondpod");
		restartMusicServices.add("com.snoggdoggler.android.applications.doggcatcher.v1_0");
		restartMusicServices.add("com.android.DroidLiveLite");
		restartMusicServices.add("com.android.DroidLiveLite:remote");
		restartMusicServices.add("com.android.DroidLivePlayer");
		restartMusicServices.add("com.android.DroidLivePlayer:remote");
		restartMusicServices.add("com.mediafly.android");
		restartMusicServices.add("com.mediafly.android.video");
		restartMusicServices.add("org.iii.ro.meridian");
		restartMusicServices.add("org.iii.ro.iiivpa");
		restartMusicServices.add("org.iii.romulus.meridian");
		restartMusicServices.add("com.mixzing.basic");
		restartMusicServices.add("com.jadn.cc");
		restartMusicServices.add("com.jadn.cc:remote");
		restartMusicServices.add("com.slacker.radio");						// Slacker Radio
		restartMusicServices.add("com.leadapps.android.radio");				// A Online Radio
		
		return restartMusicServices;
	}

	private String getOwnService() {
		// Get own set service
		SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(this);
        String ownService = settings.getString(SleepTimer.PREFS_OWN_SERVICE, "");
		return ownService;
	}

	public void setVolumeBack(int oldMusicVolumeLevel) {
		Intent newIntent = new Intent(this, SetVolumeBackReceiver.class);
		newIntent.putExtra(SetVolumeBackReceiver.INTENT_EXTRA_MUSIC_VOLUME, oldMusicVolumeLevel);
		PendingIntent sender = PendingIntent.getBroadcast(this, 0, newIntent, 0);
		Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(System.currentTimeMillis());
        calendar.add(Calendar.SECOND, 10);

        // Schedule the alarm!
        AlarmManager am = (AlarmManager)getSystemService(ALARM_SERVICE);
        am.set(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), sender);
	}
	
	public void killDeadServices() {
		Intent newIntent = new Intent(this, KillDeadServicesReceiver.class);
		PendingIntent sender = PendingIntent.getBroadcast(this, 0, newIntent, 0);
		Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(System.currentTimeMillis());
        calendar.add(Calendar.SECOND, 8);

        // Schedule the alarm!
        AlarmManager am = (AlarmManager)getSystemService(ALARM_SERVICE);
        am.set(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), sender);
	}

	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}
}
